//+------------------------------------------------------------------+
//|                                                   TrailingCT.mqh |
//|                   Copyright 2009-2013, MetaQuotes Software Corp. |
//|                                              http://www.mql5.com |
//+------------------------------------------------------------------+
#include <Math\Stat\Math.mqh>
#include <Expert\ExpertTrailing.mqh>
#include <ct_14_1s.mqh>
#define __PHI 0.61803398874989
// wizard description start
//+------------------------------------------------------------------+
//| Description of the class                                         |
//| Title=Trailing Stop based on 'Category Theory' Functors (part 1b)|
//| Type=Trailing                                                    |
//| Name=CategoryTheory                                              |
//| ShortName=CT                                                     |
//| Class=CTrailingCT                                                |
//| Page=trailing_ct                                                 |
//| Parameter=Step,double,0.5,Trailing Step                          |
//| Parameter=Consant,double,1.0,Consant Morph Functor               |
//| Parameter=Data,string,"",Ocean Data File                         |
//+------------------------------------------------------------------+
// wizard description end
//+------------------------------------------------------------------+
//| Class CTrailingCT.                                               |
//| Appointment: Class traling stops with 'Category Theory'          |
//|               Functors (part 1b).                                |
//| Derives from class CExpertTrailing.                              |
//+------------------------------------------------------------------+
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
class CTrailingCT : public CExpertTrailing
  {
protected:
   //--- adjusted parameters
   double m_step;                // trailing step
   double m_consant_morph;       // consant morph
   string m_data;                // data
   //

public:
   //--- methods of setting adjustable parameters
   
   //--- method of verification of settings
   virtual bool         ValidationSettings(void);
   //--- method of creating the indicator and timeseries
   virtual bool         InitIndicators(CIndicators *indicators);
   //--- methods of checking if the market models are formed
   virtual bool         CheckTrailingStopLong(CPositionInfo *position,double &sl,double &tp);
   virtual bool         CheckTrailingStopShort(CPositionInfo *position,double &sl,double &tp);
   //---
                        CTrailingCT(void);
                        ~CTrailingCT(void);
   //--- methods of initialization of protected data
   void                 Step(double value)                  { m_step=value;            }
   void                 Consant(double value)               { m_consant_morph=value;   }
   void                 Data(string value)                  { m_data=value;            }

protected:

      double _value;
      
      double GetOutput();
      
      string _string_value;
         
      MqlDateTime __DATETIME;
      
      void  SetOcean(int Index);
      void  SetNasdaq(int Index);
      
      CElement<string> _element_value;
      
      CMorphism<string,string> _morph_value;
      
      CCategory _category_ocean,_category_nasdaq;
      
      CDomain<string> _domain_ocean,_domain_nasdaq;
      
      CHomomorphism<string,string> _hmorph_ocean,_hmorph_nasdaq;
      
  };
//+------------------------------------------------------------------+
//| Constructor                                                      |
//+------------------------------------------------------------------+
CTrailingCT::CTrailingCT(void) : m_step(0.5),
                                 m_consant_morph(1.0),
                                 m_data("ct_14_ocean_norm_.csv")
  {
//--- initialization of protected data
      m_used_series=USE_SERIES_TIME+USE_SERIES_SPREAD+USE_SERIES_HIGH+USE_SERIES_LOW+USE_SERIES_CLOSE;
      
      _string_value="";
      
      _value=0.0;
  }
//+------------------------------------------------------------------+
//| Destructor                                                       |
//+------------------------------------------------------------------+
CTrailingCT::~CTrailingCT(void)
  {
  }
//+------------------------------------------------------------------+
//| Validation settings protected data.                              |
//+------------------------------------------------------------------+
bool CTrailingCT::ValidationSettings(void)
  {
//--- validation settings of additional filters
   if(!CExpertTrailing::ValidationSettings())
      return(false);

   if(m_period!=PERIOD_D1){
      printf(__FUNCSIG__+" Timeframe should be daily! ");
      return(false);}
//--- ok
   return(true);
  }
//+------------------------------------------------------------------+
//| Create indicators.                                               |
//+------------------------------------------------------------------+
bool CTrailingCT::InitIndicators(CIndicators *indicators)
  {
//--- check pointer
   if(indicators==NULL)
      return(false);
//--- initialization of indicators and timeseries of additional filters
   if(!CExpertTrailing::InitIndicators(indicators))
      return(false);
//--- 
//--- ok
   return(true);
  }
//+------------------------------------------------------------------+
//| Checking trailing stop and/or profit for long position.          |
//+------------------------------------------------------------------+
bool CTrailingCT::CheckTrailingStopLong(CPositionInfo *position,double &sl,double &tp)
  {
//--- check
      if(position==NULL)
         return(false);
      
      m_high.Refresh(-1);
      m_low.Refresh(-1);
      
      int _x=StartIndex();
      
      double _output=GetOutput();
      
      double _input=(m_high.GetData(_x)-m_low.GetData(_x))/m_symbol.Point();
      
      double _type=_output/fmax(1,_input+_output);
      
      double _atr=fmax(2.0*m_spread.GetData(_x)*m_symbol.Point(),m_high.GetData(_x)-m_low.GetData(_x))*(_type);
      
      double _sl=m_low.GetData(_x)-(m_step*_atr);
      
      double level =NormalizeDouble(m_symbol.Bid()-m_symbol.StopsLevel()*m_symbol.Point(),m_symbol.Digits());
      double new_sl=NormalizeDouble(_sl,m_symbol.Digits());
      double pos_sl=position.StopLoss();
      double base  =(pos_sl==0.0) ? position.PriceOpen() : pos_sl;
      
      sl=EMPTY_VALUE;
      tp=EMPTY_VALUE;
      if(new_sl>base && new_sl<level)
         sl=new_sl;
      
//---
   return(sl!=EMPTY_VALUE);
  }
//+------------------------------------------------------------------+
//| Checking trailing stop and/or profit for short position.         |
//+------------------------------------------------------------------+
bool CTrailingCT::CheckTrailingStopShort(CPositionInfo *position,double &sl,double &tp)
  {
//--- check
      if(position==NULL)
         return(false);
      
      m_high.Refresh(-1);
      m_low.Refresh(-1);
      
      int _x=StartIndex();
      
      double _output=GetOutput();
      
      double _input=(m_high.GetData(_x)-m_low.GetData(_x))/m_symbol.Point();
      
      double _type=_output/fmax(1,_input+_output);
   
      //printf(__FUNCSIG__+", input: "+DoubleToString(_input)+", output: "+DoubleToString(_output)+", type: "+DoubleToString(_type));
   
      double _atr=fmax(2.0*m_spread.GetData(_x)*m_symbol.Point(),m_high.GetData(_x)-m_low.GetData(_x))*(_type);
      
      double _sl=m_high.GetData(_x)+(m_step*_atr);
      
      double level =NormalizeDouble(m_symbol.Ask()+m_symbol.StopsLevel()*m_symbol.Point(),m_symbol.Digits());
      double new_sl=NormalizeDouble(_sl,m_symbol.Digits());
      double pos_sl=position.StopLoss();
      double base  =(pos_sl==0.0) ? position.PriceOpen() : pos_sl;
      
      sl=EMPTY_VALUE;
      tp=EMPTY_VALUE;
      if(new_sl<base && new_sl>level)
         sl=new_sl;
      
//---
      return(sl!=EMPTY_VALUE);
  }
//+------------------------------------------------------------------+
//|   Get Output value, forecast for next change in price bar range. |
//+------------------------------------------------------------------+
double CTrailingCT::GetOutput()
   {
      SetOcean(0);
      SetNasdaq(0);
      
      matrix _domain;
      vector _codomain,_inputs;
      double _output=0.0;
      
      _domain.Init(3+1,3);
      
      for(int r=0;r<4;r++)
      {
         CDomain<string> _d_new,_d_old;
         _category_ocean.GetDomain(_category_ocean.Domains()-r-1,_d_new);
         
         if(_category_ocean.Domains()-r-1-1>=0){ _category_ocean.GetDomain(_category_ocean.Domains()-r-1-1,_d_old); }
         
         for(int c=0;c<_d_new.Cardinality();c++)
         {
            CElement<string> _e_new; _d_new.Get(c,_e_new);
            
            string _s_new; _e_new.Get(0,_s_new);
            
            CElement<string> _e_old; _d_old.Get(c,_e_old);
            
            string _s_old; _e_old.Get(0,_s_old);
            
            _domain[r][c]=StringToDouble(_s_new)-StringToDouble(_s_old);
         }
      }
      
      _codomain.Init(3);
      
      for(int r=0;r<3;r++)
      {
         CDomain<string> _d_new,_d_old;
         _category_nasdaq.GetDomain(_category_nasdaq.Domains()-r-1,_d_new);
         
         if(_category_nasdaq.Domains()-r-1-1>=0){ _category_nasdaq.GetDomain(_category_nasdaq.Domains()-r-1-1,_d_old); }
         
         CElement<string> _e_new; _d_new.Get(0,_e_new);
         
         string _s_new; _e_new.Get(0,_s_new);
         
         CElement<string> _e_old; _d_old.Get(0,_e_old);
         
         string _s_old; _e_old.Get(0,_s_old);
            
         _codomain[r]=StringToDouble(_s_new)-StringToDouble(_s_old);
      }
      
      _inputs.Init(3);_inputs.Fill(m_consant_morph);
      
      M(_domain,_codomain,_inputs,_output,1);
      
      return(_output);
   }
//+------------------------------------------------------------------+
//|   Get Ocean data from file defined in input.                     | 
//|   Load it into Ocean category.                                   |
//+------------------------------------------------------------------+
void CTrailingCT::SetOcean(int Index)
   {
      //_category_ocean.Let();
      
      m_time.Refresh(-1);
      datetime _data_time=datetime(int(m_time.GetData(Index)));//-PeriodSeconds(PERIOD_D1));
      
      ResetLastError();
      int _handle=FileOpen(m_data,FILE_SHARE_READ|FILE_ANSI|FILE_COMMON,"\n",CP_ACP);
      
      int _elements=1;
      
      if(_handle!=INVALID_HANDLE)
      {
         string _line="";
         int _line_length=0;
         
         //_domain_ocean.Let();_codomain_ocean.Let();
         _hmorph_ocean.Let();
         
         while(!FileIsLineEnding(_handle))
         {
            //--- find out how many characters are used for writing the line
            _line_length=FileReadInteger(_handle,INT_VALUE);
            //--- read the line
            _line=FileReadString(_handle,_line_length);
            
            datetime _date=StringToTime(StringSubstr(_line,0,StringFind(_line,",")));
            _value=int(StringToInteger(StringSubstr(_line,StringFind(_line,",")+1,StringLen(_line))));
            
            TimeToStruct(_date,__DATETIME);
            
            if(_date>_data_time)
            {
               _category_ocean.SetDomain(_category_ocean.Domains(),_domain_ocean);
               
               //printf(__FUNCSIG__+" added domain: "+PrintDomain(_domain_ocean));
               
               //_domain_ocean=_codomain_ocean;
               
               break;
            }
            else if(__DATETIME.day_of_week!=6 && __DATETIME.day_of_week!=0 && datetime(int(_data_time)-int(_date))<=PeriodSeconds(PERIOD_D1))//_date<=_data_time && datetime(int(_data_time)-(1*PeriodSeconds(PERIOD_D1)))<=_date)
            {
               //printf(__FUNCSIG__+" on line: "+_line);
               //printf(__FUNCSIG__+" on size: "+IntegerToString(_category_ocean.Homomorphisms()));
               
               //--- print the line
               _element_value.Let();_element_value.Cardinality(1);_element_value.Set(0,DoubleToString(_value));
               _domain_ocean.Cardinality(_elements);_domain_ocean.Set(_elements-1,_element_value);
               
               //printf(__FUNCSIG__+", at: "+IntegerToString(_elements)+", adding element: "+PrintElement(_element_value)+" codomain is: "+PrintDomain(_codomain_ocean)+" domain is: "+PrintDomain(_domain_ocean));
               
               _elements++;
            }
            else
            {
            //continue;
            }
         }
         
         FileClose(_handle);
      }
      else
      {
         printf(__FUNCSIG__+" failed to load file. Err: "+IntegerToString(GetLastError()));
      }
   }
//+------------------------------------------------------------------+
//|   Get NASDAQ-100 data from symbol (NDX100 for this broker).      | 
//|   Load it into NASDAQ category.                                  |
//+------------------------------------------------------------------+
void CTrailingCT::SetNasdaq(int Index)
   {
      //_category_nasdaq.Let();
      
      m_high.Refresh(-1);
      m_low.Refresh(-1);
      
      _value=0.0;
      
      _value=(m_high.GetData(Index+StartIndex()+m_high.MaxIndex(Index,_category_ocean.Homomorphisms()))-m_low.GetData(Index+StartIndex()+m_low.MinIndex(Index,_category_ocean.Homomorphisms())))/m_symbol.Point();
      
      /*
      printf(__FUNCSIG__+
         " c value: "+DoubleToString(_value)
         );
      */
      
      _element_value.Let();_element_value.Cardinality(1);_element_value.Set(0,DoubleToString(_value));
      _domain_nasdaq.Cardinality(1);_domain_nasdaq.Set(0,_element_value);
      
      //printf(__FUNCSIG__+" date: "+TimeToString(_c_stop)+", adding value morphism: "+PrintMorphism(_morph_value));
      
      _category_nasdaq.SetDomain(_category_nasdaq.Domains(),_domain_nasdaq);
      
      //printf(__FUNCSIG__+" date: "+TimeToString(_c_stop)+", value: "+DoubleToString(_d_value,1)+", adding homomorphism: "+PrintHomomorphism(_hmorph_nasdaq));   
   }
//+------------------------------------------------------------------+
